#include "LinearGrid.h"
#define NORM
const double LinearGrid::eps = 1.19209290e-07;

// Generate Linear Grid
//LinearGrid::LinearGrid(const unsigned int _gridWidth, const unsigned int _gridHeight):gridWidth(_gridWidth), gridHeight(_gridHeight)
LinearGrid::LinearGrid(const unsigned int _gridSize):gridSize(_gridSize)
{
	texture = new Texture();
	texture->GenerateTexture(0);

	// Set Linear grid depending on width/height
	SetGrid();

	// Gridsize changed - set new
	gridSize = (unsigned int) linearGrid.size();
}

LinearGrid::~LinearGrid(void)
{

}

const Texture* LinearGrid::GetTexture() const
{
	return texture;
}


void LinearGrid::SetGrid()
{
	TexturedGrid tg;
	double v = 0.0;
	const double vTick = 1.0/(double)(gridSize);
	//double eps = 0.0;

	for (int vIndex = -(int)(gridSize/2); vIndex <= (int)(gridSize/2); vIndex++)
	{
		if(vIndex == 0)
		{
			//HACK - normalize
			#ifdef NORM
			tg.complex.imag(-eps/(gridSize/2.0));
			#else
			tg.complex.imag(-eps);
			#endif
			tg.vCoordinate = v;
			tg.isCut = true;
			SetVector(tg);
			
			//HACK - normalize
			#ifdef NORM
			tg.complex.imag(eps/(gridSize/2.0));
			#else
			tg.complex.imag(eps);
			#endif
			tg.vCoordinate = v;
			tg.isCut = false;
			SetVector(tg);
		}
		else
		{
			tg.isCut = false;
			//HACK - normalize
			#ifdef NORM
			tg.complex.imag((double)vIndex/(gridSize/2.0));
			#else
			tg.complex.imag((double)vIndex);
			#endif
			tg.vCoordinate = v;
			SetVector(tg);
		}
		
		// Increment texture coordinate value
		v += vTick;
	}
}

void LinearGrid::SetVector(TexturedGrid _tg)
{
	double u = 0.0;
	const double uTick = 1.0/(double)(gridSize);
	//double eps = 0.0;

	for (int hIndex = -(int)(gridSize/2); hIndex <= (int)(gridSize/2); hIndex++)
	{
		if (hIndex == 0)
		{
			// HACK - normalize
			#ifdef NORM
			_tg.complex.real(-eps/(gridSize/2.0));
			#else
			_tg.complex.real(-eps);
			#endif
			_tg.uCoordinate = u;
			linearVector.push_back(_tg);

			// HACK - normalize
			#ifdef NORM
			_tg.complex.real(eps/(gridSize/2.0));
			#else
			_tg.complex.real(eps);
			#endif
			_tg.uCoordinate = u;
			linearVector.push_back(_tg);
		}
		else
		{
			// HACK - normalize
			#ifdef NORM
			_tg.complex.real((double) hIndex/(gridSize/2.0));
			#else
			_tg.complex.real((double)(hIndex));
			#endif
			_tg.uCoordinate = u;
			linearVector.push_back(_tg);
		}

		u += uTick;
	}

	linearGrid.push_back(linearVector);
	linearVector.clear();
}

void LinearGrid::Draw()
{
	double x, y, u, v;

	glColor3f(0.0, 0.0, 0.0);
	glPointSize(5.0f);
	glBegin(GL_POINTS);
		glVertex2i(0,0);
	glEnd();
	glColor3f(1.0, 0.0, 0.0);
	glPointSize(1.0f);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture->GetTextureID());

	for(GridIteratorConst gridIterator = linearGrid.begin(); gridIterator != linearGrid.end() - 1; ++gridIterator)
	{
		VectorIteratorConst vItrCurrentRow = gridIterator->begin();
		VectorIteratorConst vItrUpperRow = (gridIterator+1)->begin();

		glBegin(GL_QUAD_STRIP);
		for (unsigned int i = 0; i < gridIterator->size(); i++)
		{
			// 1st/3rd/5th... Vertex for QUAD_STRIP
			x = vItrCurrentRow->complex.imag();
			y = vItrCurrentRow->complex.real();

			v = vItrCurrentRow->uCoordinate;
			u = vItrCurrentRow->vCoordinate;

			glTexCoord2f(u,v);
			glVertex2d(x,y);

			//// 2nd/4th/6th... Vertex for QUAD_STRIP
			x = vItrUpperRow->complex.imag();
			y = vItrUpperRow->complex.real();

			v = vItrUpperRow->uCoordinate;
			u = vItrUpperRow->vCoordinate;

			glTexCoord2f(u,v);
			glVertex2d(x,y);
			
			++vItrCurrentRow;
			++vItrUpperRow;

		} // end inner for
		glEnd();
	} // end outer for

	glDisable(GL_TEXTURE_2D);

}