#include "LogGrid.h"

using namespace std;


// Create complex logarithmic grid
LogGrid::LogGrid(LinearGrid* _linear, const Texture* _texture):linear(_linear), texture(_texture)
{
	// log(z) = ln|z| + i*arg(z) with -pi < arg(z) < pi
	double real, imag, absValue;
	double lnZ;
	double argZ;
	TexturedGrid tg;

	// Retrieve const-pointer to linear grid
	const ComplexGrid linGrid = linear->GetGrid();

	for(gridIterator = linGrid.begin(); gridIterator != linGrid.end(); ++gridIterator)
	{
		for(vectorIterator = gridIterator->begin(); vectorIterator != gridIterator->end(); ++vectorIterator)
		{
			real = vectorIterator->complex.real();
			imag = vectorIterator->complex.imag();

			absValue = std::abs(vectorIterator->complex);

			lnZ = std::log(absValue);
			//argZ = std::atan2(real, imag);
			argZ = std::atan2(real, imag);

			tg.uCoordinate = vectorIterator->uCoordinate;
			tg.vCoordinate = vectorIterator->vCoordinate;
			tg.isCut = vectorIterator->isCut;
			tg.complex.real(lnZ);
			tg.complex.imag(argZ);

			logVector.push_back(tg);
		}
		logGrid.push_back(logVector);
		logVector.clear();
	}
}

LogGrid::~LogGrid(void)
{
}

void LogGrid::Draw()
{
	glColor3f(0.0, 0.0, 0.0);
	glPointSize(5.0f);
	glBegin(GL_POINTS);
		glVertex2i(0,0);
	glEnd();
	glColor3f(1.0, 0.0, 0.0);
	glPointSize(1.0f);

	glBindTexture(GL_TEXTURE_2D, texture->GetTextureID());
	glEnable(GL_TEXTURE_2D);

	int tmpSize = logGrid.size()/2;
	VectorIteratorConst vItrCurrentRow;
	VectorIteratorConst vItrUpperRow;

	for(GridIteratorConst gridIterator = logGrid.begin(); gridIterator != logGrid.end() -1; ++gridIterator, tmpSize--)
	{

		vItrCurrentRow = gridIterator->begin();
		vItrUpperRow = (gridIterator+1)->begin();
		DrawHalf(vItrCurrentRow, vItrUpperRow, gridIterator->size()/1);

	} // end outer for

	glDisable(GL_TEXTURE_2D);
}			

void LogGrid::DrawHalf(VectorIteratorConst vItrCurrentRow, VectorIteratorConst vItrUpperRow, int size)
{
	double x, y, u, v;

		glBegin(GL_QUAD_STRIP);		
		for (unsigned int i = 0; i < size; i++)
		{

			if(vItrCurrentRow->isCut)
			{
				glEnd();
				glBegin(GL_QUAD_STRIP);
			}

			// 1st/3rd/5th... Vertex for QUAD_STRIP
			x = vItrCurrentRow->complex.imag()/PI;
			y = (vItrCurrentRow->complex.real() + 1.0)/1.33;

			v = vItrCurrentRow->uCoordinate;
			u = vItrCurrentRow->vCoordinate;

			glTexCoord2f(u,v);
			glVertex2d(x,y);

			//// 2nd/4th/6th... Vertex for QUAD_STRIP
			x = vItrUpperRow->complex.imag()/PI;
			y = (vItrUpperRow->complex.real() + 1.0)/1.33;

			v = vItrUpperRow->uCoordinate;
			u = vItrUpperRow->vCoordinate;

			glTexCoord2f(u,v);
			glVertex2d(x,y);
			
			++vItrCurrentRow;
			++vItrUpperRow;

			
		} // end inner for
		glEnd();
}