#include "Program.h"

const unsigned int GRID_SIZE = 50;

Program::Program(int _windowId)
{
	fileNumber = 0;

	windowId = _windowId;
	rootRes = 2*PI;

	//linearGrid = new LinearGrid(GRID_WIDTH, GRID_HEIGHT);
	linearGrid = new LinearGrid(GRID_SIZE);
	rootGrid = new RootGrid(linearGrid, linearGrid->GetTexture() );
	rootGrid->Create(rootRes);

	polygonMode = GL_FILL;

	windowWidth = glutGet (GLUT_WINDOW_WIDTH);
	windowHeight = glutGet (GLUT_WINDOW_HEIGHT);

	glClearColor (1.0, 1.0, 1.0, 1.0); // Set background 
	glShadeModel (GL_SMOOTH); // Blends colors across polygons

	glPolygonMode(GL_FRONT_AND_BACK, polygonMode);

	// Resize Projection & Viewport
	HandleReshape (windowWidth, windowHeight);
}

Program::~Program(void)
{
	
}

void Program::HandleReshape(int _windowWidth, int _windowHeight)
{
	glViewport (0, 0, _windowWidth, _windowHeight);

	InitProjection();
}

void Program::InitGlui(GLUI* _glui)
{
	glui = _glui;

	gluiWireframe = 0;
	gluiGridSize = linearGrid->GetSize();

	glui->add_statictext("Grid Options");
	glui->add_separator();

	glui->add_checkbox("Wireframe", &gluiWireframe, 1);
	glui->add_separator();

	// Radio Group
	glui->add_statictext("Select Texture: ");
	gluiRadioGroup = new GLUI_RadioGroup(glui);

	// Texture selection
	glui->add_radiobutton_to_group(gluiRadioGroup, "Galaxie.png");
	glui->add_radiobutton_to_group(gluiRadioGroup, "Cantonales.png");
	glui->add_radiobutton_to_group(gluiRadioGroup, "Treemap.png");
	glui->add_radiobutton_to_group(gluiRadioGroup, "ColorTreemap.png");

	glui->set_main_gfx_window(windowId);
}

void Program::HandleDisplay()
{
	glLoadIdentity ();

	glPolygonMode(GL_FRONT_AND_BACK, polygonMode);
	glPointSize(4.0f);

	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	glColor3f (1.0, 0.0, 0.0);

	rootGrid->Draw();
	
	glutSwapBuffers ();
}

void Program::InitProjection()
{
	glMatrixMode (GL_PROJECTION); // init scene workaround
	glLoadIdentity ();
	gluOrtho2D (-1, 1, -1, 1);
	
	glMatrixMode (GL_MODELVIEW); // init camera
}


void Program::HandleKeyboard(unsigned char key, int x, int y)
{
	switch (key)
	{
	case ESC:	exit(0);
				break;

	case 'f':	(polygonMode==GL_FILL) ? polygonMode=GL_LINE : polygonMode = GL_FILL;
				break;

	default:	break;
				// do nothing
	}

	glutPostRedisplay();
	
}

void Program::HandleSpecialKey(int key, int x, int y)
{
}

void Program::HandleMouse(int button, int state, int x, int y)
{
	oldX = x;
	oldY = y;

	if(state == GLUT_DOWN)
		mouseButton = button;

}

void Program::HandleMouseMotion(int x, int y)
{
int dy;
double weight = 0.02;

	if(mouseButton == GLUT_RIGHT_BUTTON)
	{
		dy = (oldY - y)/2;
		
		rootRes += weight * dy;

		if(rootRes >= 2*PI)
		{
			rootRes = 2*PI;
		}

		if(rootRes <= 0.01)
		{
			rootRes = 0.01;
		}
	}

	oldX = x;
	oldY = y;
}

void Program::UpdateState()
{
	if(gluiWireframe == 1)
	{
		polygonMode = GL_LINE;
	}
	else
	{
		polygonMode = GL_FILL;
	}

	if(gluiRadioGroup->get_int_val() != fileNumber)
	{
		Texture* texture = (Texture*) linearGrid->GetTexture();
		fileNumber = gluiRadioGroup->get_int_val();
		texture->GenerateTexture(fileNumber);
	}
}

void Program::HandleIdle()
{
	rootGrid->Create(rootRes);

	// Set update to GLUT window
	glutSetWindow(windowId);

	UpdateState();

	glutPostRedisplay();
}