#include "RootGrid.h"

int RootGrid::counter = 0;

RootGrid::RootGrid(LinearGrid* _linear, const Texture* _texture):gridNumber(++counter)
{
	linear = _linear;
	texture = _texture;

	Create(2*PI);
}	

void RootGrid::Create(float a)
{
	// prevent from division by zero!
	if (a == 0.0)
	{	
		return;
	}
	else
	{
		rootVector.clear();
		rootGrid.clear();
	}

	TexturedGrid tg;
	float n = (2*PI)/a; 
	openingAngle = n;
	moveUp = n;


	const ComplexGrid linGrid = linear->GetGrid();

	for(gridIterator = linGrid.begin(); gridIterator != linGrid.end(); ++gridIterator)
	{
		for(vectorIterator = gridIterator->begin(); vectorIterator != gridIterator->end(); ++vectorIterator)
		{
			//Complex nthRoot = Complex(std::pow(**itr, (double) n));
			Complex nthRoot = Complex(pow(vectorIterator->complex, 1.0/(double)n));
			nthRoot -= 1;
			nthRoot *= n;
			
			tg.uCoordinate = vectorIterator->uCoordinate;
			tg.vCoordinate = vectorIterator->vCoordinate;
			tg.isCut = vectorIterator->isCut;
			tg.complex = nthRoot;

			rootVector.push_back(tg);
		}
		rootGrid.push_back(rootVector);
		rootVector.clear();
	}

	Draw();
}


RootGrid::~RootGrid(void)
{
	linear = NULL;
	texture = NULL;
}

void RootGrid::Draw()
{
	double x, y, u, v;
	bool breakDraw = true;

	glColor3f(0.0, 0.0, 0.0);
	glPointSize(5.0f);
	glBegin(GL_POINTS);
		glVertex2i(0,0);
	glEnd();
	glColor3f(1.0, 0.0, 0.0);
	glPointSize(1.0f);

	glBindTexture(GL_TEXTURE_2D, texture->GetTextureID());
	glEnable(GL_TEXTURE_2D);

	unsigned int tmpSize = (unsigned int) rootGrid.size()/2;

	for(GridIteratorConst gridIterator = rootGrid.begin(); gridIterator != rootGrid.end() - 1; ++gridIterator)
	{

		VectorIteratorConst vItrCurrentRow = gridIterator->begin();
		VectorIteratorConst vItrUpperRow = (gridIterator+1)->begin();

		static double openY = 1.0;
		
		glBegin(GL_QUAD_STRIP);
		for (unsigned int i = 0; i < gridIterator->size(); i++)
		{

			if(vItrCurrentRow->isCut && (i <= tmpSize))
			{
				glEnd();
				glBegin(GL_QUAD_STRIP);
			}

			if(moveUp > 2.0)
			{
				moveUp = 2.0;
			}

			if(openingAngle > PI) 
			{
				openingAngle = PI;
			}

			//printf("openY: %f\n", openY);

			// 1st/3rd/5th... Vertex for QUAD_STRIP
			x = vItrCurrentRow->complex.imag()/openingAngle;
			y = (vItrCurrentRow->complex.real() + moveUp)/openingAngle;

			u = vItrCurrentRow->uCoordinate;
			v = vItrCurrentRow->vCoordinate;

			glTexCoord2f(u,v);
			glVertex2d(x,y);

			//// 2nd/4th/6th... Vertex for QUAD_STRIP
			x = vItrUpperRow->complex.imag()/openingAngle;
			y = (vItrUpperRow->complex.real() + moveUp)/openingAngle;

			u = vItrUpperRow->uCoordinate;
			v = vItrUpperRow->vCoordinate;

			glTexCoord2f(u,v);
			glVertex2d(x,y);

			++vItrCurrentRow;
			++vItrUpperRow;

		} // end inner for
		glEnd();
	} // end outer for

	glDisable(GL_TEXTURE_2D);
}