#include "Texture.h"

Texture::Texture()
{
	fileNames.push_back("galaxie.png");
	fileNames.push_back("cantonales.png");
	fileNames.push_back("treemap.png");
	fileNames.push_back("ctreemap.png");
}

Texture::~Texture(void)
{
}

void Texture::GenerateTexture(int fileNumber)
{
	bool result = LoadPNG (fileNames[fileNumber].c_str());

	if(!result)
	{
		printf("Texture file not found ... exiting\n");
		exit(-1);
	}

	glDeleteTextures(1, &textureId);

	glGenTextures(1, &textureId);
	glBindTexture(GL_TEXTURE_2D, textureId);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	GLsizei w = (GLsizei)this->textureWidth;
	GLsizei h = (GLsizei)this->textureHeight;

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, this->pngImage);
	free(pngImage);
}

GLuint Texture::GetTextureID () const
{
	return textureId;
}

bool Texture::LoadPNG(const char *filename)
{
	//char *fname = "cantonales.png";
	
	// PNG Structures
	unsigned char header[8];
	png_structp pngPtr;
	png_infop pngInfoPtr;
	png_bytepp rowPointers = NULL;

	int bitDepth;
	int colorType;

	unsigned int rowbytes;
	char* imageData = NULL;

	// End of PNG Structures
	FILE *fptr = fopen(filename, "rb");

	if(!fptr)
	{
		return false;
	}

	fread(header, 1, 8, fptr);
	if(!png_check_sig((unsigned char*) header, 8))
	{
		fclose (fptr);
		return false;
	}

	pngPtr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);

	if (!pngPtr)
	{
		fclose(fptr);
		return false;
	}

	pngInfoPtr = png_create_info_struct(pngPtr);

	if (!pngInfoPtr)
	{
		png_destroy_read_struct(&pngPtr, (png_infopp) NULL, (png_infopp) NULL);
		fclose (fptr);
		return false;
	}

	if (setjmp(png_jmpbuf(pngPtr)))
	{
		png_destroy_read_struct (&pngPtr, &pngInfoPtr, NULL);
		fclose(fptr);
		return false;
	}

	png_init_io(pngPtr, fptr);
	png_set_sig_bytes(pngPtr, 8);
	png_read_info(pngPtr, pngInfoPtr);
	png_get_IHDR(pngPtr, pngInfoPtr, &textureWidth, &textureHeight, &bitDepth, &colorType, NULL, NULL, NULL);

	if(colorType & PNG_COLOR_MASK_ALPHA)
	{
		png_set_strip_alpha(pngPtr);
	}
	
	if (bitDepth > 8)
	{
		png_set_strip_16(pngPtr);
	}

	if (colorType == PNG_COLOR_TYPE_GRAY || colorType == PNG_COLOR_TYPE_GRAY_ALPHA)
	{
		png_set_gray_to_rgb(pngPtr);
	}

	if (colorType == PNG_COLOR_TYPE_PALETTE)
	{
		png_set_palette_to_rgb(pngPtr);
	}

	png_read_update_info(pngPtr, pngInfoPtr);
	rowbytes = png_get_rowbytes(pngPtr, pngInfoPtr);

	if((imageData = (char*)malloc(rowbytes * textureHeight)) == NULL)
	{
		png_destroy_read_struct(&pngPtr, &pngInfoPtr, NULL);
		free(imageData);
		imageData=NULL;
		return false;
	}

	if((rowPointers = (png_bytepp)malloc(textureHeight*sizeof(png_bytep))) == NULL)
	{
		png_destroy_read_struct(&pngPtr, &pngInfoPtr, NULL);
		free(imageData);
		imageData = NULL;
		return false;
	}

	for (unsigned int i = 0; i < textureHeight; ++i)
	{
		//HACK
		rowPointers[textureHeight - 1 - i] = (png_bytep)(imageData + i*rowbytes);
	}

	png_read_image(pngPtr, rowPointers);
	free(rowPointers);
	png_destroy_read_struct(&pngPtr, &pngInfoPtr, NULL);
	fclose(fptr);

	pngImage = imageData;

	return true;
}